#!/usr/bin/perl -wT
use strict;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: browse.cgi,v $
## Revision 1.4  2002/10/14 15:30:08  grumpy
## changes to 0
##
## Revision 1.3  2002/10/13 16:57:39  grumpy
##
## Added 2002 copyright notice.
##
##
####

BEGIN {
    $ENV{'PATH'} = '/bin;/usr/bin;/usr/local/bin';
    $| = 1;
    push @INC, '.';
    require Kumera;
    Kumera->import(qw/:DEFAULT :SESSION :DEBUG/);
    push @INC, $Kumera::libs;
}

use CGI qw/:standard/;
use CGI::Carp qw/fatalsToBrowser/;

use HTML::TreeBuilder;
use URI;

# redefine $DEBUG, check that it is the same as the paramater from the URL
$DEBUG = $DEBUG && param('debug') eq $DEBUG;

print header(-type=>'text/plain') if $DEBUG;

# put this after the debug setting so we can grab its output
use Kumera::Browse;

my ($path_info, $session_id) = split /;/, path_info();
$path_info = $path_info && $path_info =~ /^([A-Za-z0-9\/_.]+)$/ ? $1 : '';
$session_id = '' unless $session_id;

my ($user_id, $last_activity) = @{ decode($session_id) };
$user_id = '' unless $user_id;
$last_activity = '' unless $last_activity;

# build a new $session_id
$session_id = encode([$user_id, time()]);

# now make the browse object
my $browse = new Kumera::Browse({ PATH_INFO => $path_info, USER_ID => $user_id });

# fix path_info
$path_info = $browse->getPathInfo();

# check if the user is valid (will have more processing eventually)
if
(
    !$browse->getName($user_id) ||
    $last_activity !~ /^\d+$/ ||
    $last_activity + $browse->getParameter('timeout') < time() ||
    !$browse->hasPermission('browse')
)
{
    print redirect(-uri => $script_path.'/login.cgi'.$path_info.';'.$session_id);
    exit;
}

####
##
## From here it is uniquely browse.cgi
##
####

# parse in the file that we need
my $htmlTree = new HTML::TreeBuilder;

$htmlTree->parse_file($browse->getBrowseSource);

my $script_url = new URI('http://'.server_name().script_name());
my $target_url = new URI('http://'.server_name().$path_info);

# helper code, which things get rewirtten URLs and which don't
my %links =
    (
        a => ['href', 1],
        img => ['src', 0],
        form => ['action', 0],
        link => ['href', 0],
        frame => ['src', 1],
    );

my @javascript =
    (
        'onmouseover',
        'onmouseout',
    );

my @not_really =
    (
        'pdf',
    );

# and then do the traverse

$htmlTree->traverse
            (
                sub {

                    my $element = shift;
                    my $start  = shift;

                    # if the start of a tag, then process it for URL translation
                    return 1 unless $start;

                    my $attr = $links{ lc $element->tag() } or return 1;
                    my $relative_url = $element->attr($attr->[0]) or return 1;

                    # skip out if url is not what we want (eg. pdf, etc)
                    return 1 if grep { $relative_url =~ /\.($_)$/; } @not_really;

                    # build an absolute url from the realtive one, with target as the base
                    my $absolute_url = new_abs URI($relative_url, $target_url);

                    # and change the attribute to what we want
                    $element->attr($attr->[0], (($attr->[1] && ($absolute_url->scheme eq 'http' && $absolute_url->host eq server_name())) ? $script_url.$absolute_url->path.';'.$session_id : $absolute_url->as_string));

                    # now strip out the javascript event handlers
                    map { $element->attr($_, 'return false;') if $element->attr($_) } @javascript;

                    return 1;
                },
            1
    );

# and throw in the html, etc, yada, yada
%HTML::Element::optionalEndTag = map { $_ => 1 } qw(dt dd);
my $html = $htmlTree->as_HTML;

$htmlTree->delete;

# put the menu at the end ...

my $message;
my $edit_bool = 0;

# check if the file is locked out by someone
if ($browse->is_locked_by) {

    # this use has the file locked out
    $message = 'You currently have this file locked out.';
    $edit_bool = 1;

} else {

    my $other_user = $browse->locked_out_by;

    $browse->_debug('other_user', $other_user);

    # may not be locked out
    if ($other_user) {

        $message = 'This file is locked out by '.
                a(
                    {
                        -href => 'mailto:'.$browse->getEmail($other_user).'?subject=You%20have%20'.$path_info.'%20locked%20out%20...&body=You%20currently%20have%20the%20file%20'.$path_info.'%20locked%20out. '.$browse->getName($user_id).'%20would%20like%20to%20edit%20it,%20could%20you%20have%20it%20commited%20in%20please.'."%20\nThanks%20...%20\n\nThe%20System."
                    },
                    $browse->getName($other_user)
                )

    }

    # may not be lockable (has no .xml file at all)
    $edit_bool = $browse->is_lockable;
}

my $menu = $browse->menu({ create => 1, edit => $edit_bool, delete => $browse->is_deleteable, commit => 1, upload => 1, logout => 1 }, $path_info.';'.$session_id, $message);
$html =~ s,(</body> *</html>),$menu$1,m;

print header, $html;

