#!/usr/bin/perl -wT
use strict;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: create.cgi,v $
## Revision 1.3  2002/10/14 15:30:08  grumpy
## changes to 0
##
## Revision 1.2  2002/10/13 16:57:39  grumpy
##
## Added 2002 copyright notice.
##
##
####

BEGIN {
    $ENV{'PATH'} = '/bin;/usr/bin;/usr/local/bin';
    $| = 1;
    push @INC, '.';
    require Kumera;
    Kumera->import(qw/:DEFAULT :SESSION :DEBUG/);
    push @INC, $Kumera::libs;
}

use CGI qw/:standard/;
use CGI::Carp qw/fatalsToBrowser/;

use File::Basename;

# redefine $DEBUG, check that it is the same as the paramater from the URL
$DEBUG = $DEBUG && param('debug') eq $DEBUG;

print header(-type=>'text/plain') if $DEBUG;

# put this after the debug setting so we can grab its output
use Kumera::Create;

my ($path_info, $session_id) = split /;/, path_info();
$path_info = $path_info && $path_info =~ /^([A-Za-z0-9\/_.]+)$/ ? $1 : '';
$session_id = '' unless $session_id;

my ($user_id, $last_activity) = @{ decode($session_id) };
$user_id = '' unless $user_id;
$last_activity = '' unless $last_activity;

# build a new $session_id
$session_id = encode([$user_id, time()]);

# now make the "create" object
my $create = new Kumera::Create({ PATH_INFO => $path_info, USER_ID => $user_id });

# fix path_info
$path_info = $create->getPathInfo();

# check if the user is valid (will have more processing eventually)
if
(
    !$create->getName($user_id) ||
    $last_activity !~ /^\d+$/ ||
    $last_activity + $create->getParameter('timeout') < time() ||
    !$create->hasPermission('create')
)
{
    print redirect(-uri => $script_path.'/login.cgi'.$path_info.';'.$session_id);
    exit;
}



####
##
## From here it is uniquely create.cgi
##
####

my $message = "Please enter a filename to create.";

if (defined param('action') && param('action') eq 'do_create') {

    # untaint and neaten it up (hmm, ../../../.. is still valid)
    my $create_filename = param('filename') =~ /^([A-Za-z0-9\/_.-]+)/ ? lc $1 : '';

    $create->_debug('filename', $create_filename);
    $create->_debug('default', (dirname($path_info) ne '/' ? dirname($path_info) : '').$create->getCreateFilename('default'));
    $create->_debug('permission', $create->hasPermission('create', $create_filename));
    $create->_debug('exists', $create->getCreateFilename('root').$create_filename);
    $create->_debug('locked_out_by', $create->locked_out_by({ PATH_INFO => $create_filename}));

    # first check if the filename has been entered
    if (!defined $create_filename || !$create_filename) {

        $message = "Please enter a filename.";

    # we won't accept the default name at all
    } elsif (defined $create_filename && $create_filename eq (dirname($path_info) ne '/' ? dirname($path_info) : '').$create->getCreateFilename('default')) {

        $message = "Please enter a filename other then the default one given.";

    # check we have permission to create this file
    } elsif (!$create->hasPermission('create', $create_filename)) {

        $message = "You do not have permission to create that file.";

    # now check the file doesn't already exist, or is locked out
    } elsif (-w $create->getCreateFilename('root').$create_filename || $create->locked_out_by({ PATH_INFO => $create_filename})) {

        $message = "That file already exists.";

    # filename must end in html
    } elsif ($create_filename !~ /\.html$/) {

        $message = "The filename must end in '.html'";

    # no errors, we can write this to disk
    } else {

        $create->createFile($create_filename);

        # and then jump to edit to start working on it, with new
        # filename (not old path_info)
        print redirect(-uri => $script_path.'/edit.cgi'.$create_filename.';'.$session_id);
        exit;
    }
}


# clean out filename in case this is a loop back, and a new one needs to be entered
my $filename = (dirname($path_info) ne '/' ? dirname($path_info) : '').$create->getCreateFilename('default');
param(-name => 'filename', -value => $filename);

print
    header,
    start_html(-title => 'create'),
    start_form(-action => script_name().$path_info.';'.$session_id),
    hidden(-name => 'action', -value => 'do_create'),
    table(
        Tr({-bgcolor => '#ffffff'},
            [
                td({-valign => 'top', -colspan => 2}, b($message)),
                td({-valign => 'top'},
                    [
                        b('filename'),
                        textfield(-name => 'filename', -size => 40),
                    ]
                ),
                td({-valign => 'top', -align => 'right', -colspan => 2}, submit(-value => 'create')),
            ]
        )
    ),
    end_form,
    $create->menu({ browse => 1, logout => 1 }, $path_info.';'.$session_id),
    end_html;


