#!/usr/bin/perl -wT
use strict;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: delete.cgi,v $
## Revision 1.3  2002/10/14 15:30:08  grumpy
## changes to 0
##
## Revision 1.2  2002/10/13 16:57:39  grumpy
##
## Added 2002 copyright notice.
##
##
####

BEGIN {
    $ENV{'PATH'} = '/bin;/usr/bin;/usr/local/bin';
    $| = 1;
    push @INC, '.';
    require Kumera;
    Kumera->import(qw/:DEFAULT :SESSION :DEBUG/);
    push @INC, $Kumera::libs;
}

use CGI qw/:standard/;
use CGI::Carp qw/fatalsToBrowser/;

use File::Basename;

# redefine $DEBUG, check that it is the same as the paramater from the URL
$DEBUG = $DEBUG && param('debug') eq $DEBUG;

print header(-type=>'text/plain') if $DEBUG;

# put this after the debug setting so we can grab its output
use Kumera::Delete;

my ($path_info, $session_id) = split /;/, path_info();
$path_info = $path_info && $path_info =~ /^([A-Za-z0-9\/_.]+)$/ ? $1 : '';
$session_id = '' unless $session_id;

my ($user_id, $last_activity) = @{ decode($session_id) };
$user_id = '' unless $user_id;
$last_activity = '' unless $last_activity;

# build a new $session_id
$session_id = encode([$user_id, time()]);

# now make the "delete" object
my $delete = new Kumera::Delete({ PATH_INFO => $path_info, USER_ID => $user_id });

# fix path_info
$path_info = $delete->getPathInfo();

# check if the user is valid (will have more processing eventually)
if
(
    !$delete->getName($user_id) ||
    $last_activity !~ /^\d+$/ ||
    $last_activity + $delete->getParameter('timeout') < time() ||
    !$delete->hasPermission('delete')
)
{
    print redirect(-uri => $script_path.'/login.cgi'.$path_info.';'.$session_id);
    exit;
}

####
##
## From here it is uniquely delete.cgi
##
####

# check they have permission to delete this file first
# can't delete index file no matter what!
unless (!$delete->isIndexFile && ($delete->is_deleteable || $delete->hasPermission('delete.section'))) {
    print redirect(-uri => $script_path.'/browse.cgi'.$path_info.';'.$session_id);
    exit;
}

my $message = "Enter the file you want to delete.";

if (defined param('action') && param('action') eq 'do_delete') {

    # untaint and neaten it up (hmm, ../../../.. is still valid)
    my $delete_filename = param('filename') =~ /^([A-Za-z0-9\/_.-]+)/ ? lc $1 : '';
    my $edit_filename = $delete_filename =~ s/\.xml$/.html/;

    $delete->_debug('filename', $delete_filename);
    $delete->_debug('permission', $delete->hasPermission('delete', $delete_filename));
    $delete->_debug('exists', $delete->getDeleteFilename('root').$delete_filename);
    $delete->_debug('locked_out_by', $delete->locked_out_by({ PATH_INFO => $delete_filename}));

    # first check if the filename has been entered
    if (!defined $delete_filename || !$delete_filename) {

        $message = "Please enter a filename.";

    # filename must end in html
    } elsif ($delete_filename !~ /\.html$/) {

        $message = "The filename must end in '.html'";

    # now check the file exists (it will as an .xml file)
    } elsif (!-w $delete->getDeleteFilename('root').$edit_filename) {

        $message = "That file does not exist.";

    # no errors, we can write this to disk
    } else {

        $delete->deleteFile($delete_filename);

        # and then jump to browse of the index file for the next thing to do
        print redirect(-uri => $script_path.'/browse.cgi'.(dirname($path_info) ne '/' ? dirname($path_info) : '').$delete->getBrowseFilename('index').';'.$session_id);
        exit;
    }
}


# put the path info into this for the default
param(-name => 'filename', -value => $path_info);

print
    header,
    start_html(-title => 'delete'),
    start_form(-action => script_name().$path_info.';'.$session_id),
    hidden(-name => 'action', -value => 'do_delete'),
    table(
        Tr({-bgcolor => '#ffffff'},
            [
                td({-valign => 'top', -colspan => 2}, b($message)),
                td({-valign => 'top'},
                    [
                        b('filename'),
                        textfield(-name => 'filename', -size => 40),
                    ]
                ),
                td({-valign => 'top', -align => 'right', -colspan => 2}, submit(-value => 'delete')),
            ]
        )
    ),
    end_form,
    $delete->menu({ browse => 1, logout => 1 }, $path_info.';'.$session_id),
    end_html;


