#!/usr/bin/perl -wT
use strict;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: edit.cgi,v $
## Revision 1.3  2002/10/14 15:30:08  grumpy
## changes to 0
##
## Revision 1.2  2002/10/13 16:57:39  grumpy
##
## Added 2002 copyright notice.
##
##
####

BEGIN {
    $ENV{'PATH'} = '/bin;/usr/bin;/usr/local/bin';
    $| = 1;
    push @INC, '.';
    require Kumera;
    Kumera->import(qw/:DEFAULT :SESSION :DEBUG/);
    push @INC, $Kumera::libs;
}

use CGI qw/:standard/;
use CGI::Carp qw/fatalsToBrowser/;

# redefine $DEBUG, check that it is the same as the paramater from the URL
$DEBUG = $DEBUG && param('debug') eq $DEBUG;

print header(-type => 'text/plain') if $DEBUG;

# put this after the debug setting so we can grab its output
use Kumera::Edit;

my ($path_info, $session_id) = split /;/, path_info();
$path_info = $path_info && $path_info =~ /^([A-Za-z0-9\/_.]+)$/ ? $1 : '';
$session_id = '' unless $session_id;

my ($user_id, $last_activity) = @{ decode($session_id) };
$user_id = '' unless $user_id;
$last_activity = '' unless $last_activity;

# build a new $session_id
$session_id = encode([$user_id, time()]);

# and make the edit object, just for fun
my $edit = new Kumera::Edit({ PATH_INFO => $path_info, USER_ID => $user_id });

# fix path_info
$path_info = $edit->getPathInfo;

# check if the user is valid (will have more processing eventually)
if
(
    !$edit->getName($user_id) ||
    $last_activity !~ /^\d+$/ ||
    $last_activity + $edit->getParameter('timeout') < time() ||
    !$edit->hasPermission('edit')
)
{
    print redirect(-uri => $script_path.'/login.cgi'.$path_info.';'.$session_id);
    exit;
}

####
##
## From here it is uniquely login.cgi
##
####

# check if the file we have been given can be edited, this will include
# being locked out by another user (not this one)

# must either be not locked out, or locked out to this user_id
# might be locked out, and this person has permission to edit it
# this stops people from entering edit.cgi url directly
if ($edit->isIndexFile || !$edit->is_lockable) {
    print redirect(-uri => $script_path.'/browse.cgi'.$path_info.';'.$session_id);
    exit;
}

# now lock the file out, this will read in the file, parse it
# and prepare it for writing out if required
$edit->makeLockfile;

# we are either saving the content away, or viewing it
if (defined param('action') && param('action') eq 'do_save') {

    # put new contents in
    foreach my $field (param('field')) {
        $edit->putText($field, param($field));
    }

    # save the file away
    $edit->save;

    # and redirect back to browse, which should reload the data
    # for only this user to view
    print redirect(-uri => $script_path.'/browse.cgi'.$path_info.';'.$session_id);

} else {

    # build the list of names for the edit fields
    my $table_rows = '';
    foreach my $name ($edit->getNames()) {

        my $row;

        $edit->getType($name) eq 'textarea' and $row = textarea(-name => $name, -default => $edit->getText($name), -rows => 30, -columns => 75, -wrap => 'virtual');
        $edit->getType($name) eq 'textfield' and $row = textfield(-name => $name, -value => $edit->getText($name), -size => 40);
        $edit->getType($name) eq 'hidden' and $row = hidden(-name => $name, -value => $edit->getText($name)).$edit->getText($name);


        $table_rows .= Tr
                        ({-bgcolor => '#ffffff'},
                        [
                            td({-valign => 'top'},
                            [
                                b($name).hidden(-name => 'field', -value => $name),
                                $row
                            ]),
                        ]);

    }

    $table_rows .= Tr({-bgcolor => '#ffffff'}, [td({-valign => 'top', -align => 'right', -colspan => 2}, [submit(-value => '  save  ')])]);

    print
            header,
            start_html(-title => 'edit'),
            start_form(-action => script_name().$path_info.';'.$session_id),
            hidden(-name => 'action', -value => 'do_save'),
            table($table_rows),
            end_form,
            $edit->menu({browse => 1, logout => 1}, $path_info.';'.$session_id),
            end_html;

}


