package Kumera::Commit;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Commit.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

use File::Find;
use File::Basename;
use File::Copy;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ %{ $args }});

    return bless $this, $type;
}


sub getCommitList {

    my $this = shift;

    # for output
    my @commit_list;

    # cache here
    my $root_path = $this->getEditFilename('root');
    my $user_id = $this->{ USER_ID };
    my $this_user_id;

    my $section_id;
    my $this_section_id;

    my $path_info;
    my $html_filename;
    my $this_filename;

    $this->_debug('user_id', $user_id);
    $this->_debug('root_path', $root_path);

    # look for files that we have checked out, and have permission to check it
    find(
            {
                wanted => sub {

                    $this->_debug('found', $_);

                    # skip this unless it is exactly what we want
                    # we want xml files of the form, user_id.section_id.xml
                    # which depends on the section and user_id and acl/roles/groupings
                    return unless /^(.*?)\.(.*?)\.(.*?)\.xml$/;

                    # see if we want this file $_ has the current filename in this dir
                    $this_filename = $1;
                    $this_user_id = $2;
                    $this_section_id = $3;

                    $this->_debug('this_filename', $this_filename);
                    $this->_debug('this_user_id', $this_user_id);
                    $this->_debug('this_section_id', $this_section_id);

                    # skip if an index file
                    return if $this_filename eq 'index';

                    # strip the leading directory information from the front
                    $File::Find::name =~ m/^$root_path(.*)\.$this_user_id\.$this_section_id\.xml$/;
                    $path_info = "$1.html";
                    $html_filename = "$1.$this_user_id.$this_section_id.html";

                    $this->_debug('File::Find::name', $File::Find::name);
                    $this->_debug('path_info', $path_info);

                    # (1) have they got commit.section priv. for this file's path_info
                    # (2) does this_section_id match the user's section_id commit.section's id
                    # or
                    # (1) have they got commit.user priv. for this section
                    # (2) does this_user_id match their user_id
                    if
                    (
                        (
                            $this->hasPermission('commit.section', $path_info)
                            &&
                            $this->_get_section_id($path_info) eq $this_section_id
                        )
                        ||
                        (
                            $this->hasPermission('commit.user', $path_info)
                            &&
                            $this_user_id eq $user_id
                        )
                    )
                    {
                        push @commit_list, [ $html_filename, $path_info, $this_user_id, $this_section_id ];

                    }

                },
                untaint => 1,
                bydepth => 1
            },
            $root_path
        );

    return sort { $a->[0] cmp $b->[0]   } @commit_list;
}

sub processCommitList {

    my $this = shift;
    my @commit_list = @_;

    # for return
    my @complete_list = ();

    # loop through the files that have been selected to be checked back in
    my $commit_root = $this->getCommitFilename('root');

    my $commit_filename;
    my $path_info;
    my $lock_filename;
    my $edit_filename;

    # loop through the given commit_list
    foreach my $html_filename (@commit_list) {

        # path_info may be tained, so we clean it up here
        # this needs to be more exhaustive as /../../../../../etc/passwd becomes valid
        $html_filename = $html_filename =~ /^([A-Za-z0-9\/_.-]+)$/ ? $1 : '';

        # need to pre-pend the edit root to make a correct html_filename
        $html_filename = $this->getEditFilename('root').$html_filename;

        # we have been given the html_filename, with user_id and section_id
        # built in, so we don't need to do much processing with it
        $path_info = $this->_build_commit_filename({ HTML_FILENAME => $html_filename });
        $commit_filename = $commit_root.$path_info;

        $this->_debug('html_filename', $html_filename);
        $this->_debug('commit_filename', $commit_filename);
        $this->_debug('path_info', $path_info);

        # skip this if we can't read it correctly
        next unless -w $html_filename;

        # move the html_filename over to the commit_filename, and make sure permisions are okay
        move $html_filename, $commit_filename or die "Unable to move $html_filename to $commit_filename: $!";
        chmod $Kumera::file_perms, $commit_filename;

        # then uncommit the lockfile back to it's natural state
        ($lock_filename = $html_filename) =~ s/\.html$/.xml/;
        $edit_filename = $this->_build_edit_filename({ PATH_INFO => $path_info });
        die "cannot unlock $lock_filename: $!" unless move $lock_filename, $edit_filename;

        push @complete_list, $path_info;
    }

    # rebuild all the index files for the users
    $this->_rebuild_index_files(\@complete_list);

    return @complete_list;
}

1;
