package Kumera::Create;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Create.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ %{ $args }});

    # and now set the template filename for create, this will overload the
    # Kumera.pm one (which is actually used by edit)
    $this->{ CREATE_TEMPLATE_FILENAME } = $this->_build_template_filename;
    $this->_debug('this->{ CREATE_TEMPLATE_FILENAME }', $this->{ CREATE_TEMPLATE_FILENAME });

    return bless $this, $type;
}

use File::Basename;
use File::Copy;

sub _build_template_filename {

    my $this = shift;
    my $args = shift;

    my $user_id = $args->{ USER_ID } || $this->{ USER_ID };
    my $path_info = $args->{ PATH_INFO } || $this->{ PATH_INFO };

    # need to loop through the directory structure
    # looking for the template that we want to use
    my $template_path = $path_info;

    while ($template_path ne '/' && $template_path ne '.') {

        $template_path = dirname($template_path);

        $template_filename = $this->getCreateFilename('root').($template_path ne '/' ? $template_path : '').$this->getCreateFilename('template');

        $this->_debug('template_filename', $template_filename);

        last if -r $template_filename;
    }

    $this->{ TEMPLATE_PATH_INFO } = $template_path.$this->getCreateFilename('template');

    return $template_filename;
}

# assume we have a good filename to work with (create.cgi does checks)
sub createFile {

    my $this = shift;
    my $create_filename = shift;

    # build the destination filename
    $create_filename = $this->getCreateFilename('root').$create_filename;
    $create_filename =~ s/\.html$/.xml/;

    $this->_debug('create_filename', $create_filename);
    $this->_debug('template_filename', $template_filename);

    # and do the copy
    copy $this->{ CREATE_TEMPLATE_FILENAME }, $create_filename;

    # and change permissions
    chmod $Kumera::file_perms, $create_filename;
}

1;

