package Kumera::Delete;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Delete.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ %{ $args }});

    return bless $this, $type;
}

# assume we have a good filename to work with (delete.cgi does checks)
sub deleteFile {

    my $this = shift;
    my $path_info = shift;

    # either the file is locked out, or it is not locked out
    # assume if they are this far, then they are allowed to delete the file

    my $locked_out_by = $this->locked_out_by({ USER_ID => $user_id, PATH_INFO => $path_info });

    # might be two files to delete, the xml (always) and html (is locked out)
    my $html_filename = '';
    if ($locked_out_by) {

        # this html file
        $html_filename = $this->_build_html_filename({ USER_ID => $locked_out_by, PATH_INFO => $path_info });

        # the xml file
        $delete_filename = $this->_build_lock_filename({ USER_ID => $locked_out_by, PATH_INFO => $path_info });

    } else {

        # build the other filename
        $delete_filename = $this->_build_edit_filename({ USER_ID => $locked_out_by, PATH_INFO => $path_info });

    }

    $this->_debug('path_info', $path_info);
    $this->_debug('delete_filename', $delete_filename);
    $this->_debug('html_filename', $html_filename);

    # and rebuild the index file (before we delete the xml and html file)
    $this->_rebuild_index_files([ $path_info ], 1);

    # and do the delete of the xml file
    unlink $delete_filename if -w $delete_filename;
    unlink $html_filename if -w $html_filename;
}

1;

