package Kumera::Edit;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Edit.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

use XML::DOM;
use IO::File;
use File::Basename;
use File::Copy;

use constant NAME => 0;
use constant TYPE => 1;
use constant TEXT => 2;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ %{ $args }});

    # next do the template filename
    $this->{ TEMPLATE_FILENAME } = $this->_build_template_filename;

    $this->_debug('this->{ TEMPLATE_FILENAME }', $this->{ TEMPLATE_FILENAME });

    return bless $this, $type;
}

sub _build_template_filename {

    my $this = shift;
    my $args = shift;

    my $user_id = $args->{ USER_ID } || $this->{ USER_ID };
    my $path_info = $args->{ PATH_INFO } || $this->{ PATH_INFO };

    return dirname($this->_build_edit_filename).'/'.$this->getEditFilename('template');
}

# client functions
sub _make_lockfile {

    my $this = shift;

    # it is either (1) locked out to this user or
    # (2) locked out to another user and this user has edit.section
    # permission (which is taken care of in building the lock_filename)
    return $this->{ LOCK_FILENAME } if -w $this->{ LOCK_FILENAME };

    # make lock file (this includes expectation of it being locked out by another user)
    die "Cannot create lock file: $!" unless move $this->{ EDIT_FILENAME }, $this->{ LOCK_FILENAME };

    # the expected one
    return $this->{ LOCK_FILENAME };
}

sub _read_htmlfile {

    my $this = shift;

    # read in the template file
    my $input = new IO::File($this->{ TEMPLATE_FILENAME });
    $input->untaint;
    my $template = join '', $input->getlines;
    $input->close;

    # build the hash that template expects
    my %text = ();
    foreach my $name ($this->getNames) {
        $text{ $name } = $this->getText( $name );
    }

    # now search for the [[ ]] bits to parse and execute
    $template =~ s/\[\[(.*?)\]\]/$1/eesg;

    return $template;
}

sub _make_htmlfile {

    my $this = shift;

    my $template = $this->_read_htmlfile;

    # and save the html away to disk, regardless
    # of who locked it out, and who we are as html_filename
    # will have the correct name to save to

    my $output = new IO::File($this->{ HTML_FILENAME }, O_CREAT|O_TRUNC|O_WRONLY, 0666);
    $output->write($template, length $template);
    $output->close;

    # because IO::File doesn't seem to do it right ...
    chmod $Kumera::file_perms, $this->{ HTML_FILENAME };
}

sub makeLockfile {

    my $this = shift;

    # make the lockfile first (if required), it will return the
    # *real* lock_filename back to us
    my $lock_filename = $this->{ LOCK_FILENAME };
    $lock_filename = $this->_make_lockfile if $this->is_lockable && !defined $this->{ READ_ONLY };

    # now read in the lockfile and make the DOMTree
    my $DOMTree = new XML::DOM::Parser->parsefile($lock_filename);

    # keep this for when we do output
    $this->{ EDIT_DOM_TREE } = $DOMTree;

    # now go through the DOMTree and extract into a hash
    # the information we really need (and push in url as create
    # does not put it there, or it may have changed)
    $this->{ EDIT_DATA } = Kumera::Base::_parse_edit_document($DOMTree, ['url', $this->{ PATH_INFO }]);

    # and if it is read only here
    return if defined $this->{ READ_ONLY };

    # now make the htmlfile from this document hash
    $this->_make_htmlfile;

    # and make the new index page for them
    $this->_rebuild_indexfile;
}


sub getText {

    my $this = shift;
    my $name = shift;

    foreach my $content (@{ $this->{ EDIT_DATA }}) {
        return $content->[ TEXT ] if $content->[ NAME ] eq $name;
    }

    return '';
}

sub getType {

    my $this = shift;
    my $name = shift;

    foreach my $content (@{ $this->{ EDIT_DATA }}) {
        return $content->[ TYPE ] if $content->[ NAME ] eq $name;
    }

    return '';
}

sub putText {

    my $this = shift;
    my $name = shift;
    my $text = shift;

    # put into document tree
    my $DOMTree = $this->{ EDIT_DOM_TREE };
    $this->{ EDIT_DATA } = Kumera::Base::_parse_edit_document($DOMTree, [$name, $text]);
}

sub getNames {

    my $this = shift;

    my @names = map { $_->[ NAME ] } @{ $this->{ EDIT_DATA }};

    $this->_debug('names', join ', ', @names);

    return @names;
}

sub save {

    my $this = shift;

    # first save the DOMTree to disk
    $this->{ EDIT_DOM_TREE }->printToFile($this->{ LOCK_FILENAME });
    chmod $Kumera::file_perms, $this->{ LOCK_FILENAME };

    # and then make the new htmlfile
    $this->_make_htmlfile;

    # and rebuild the index file
    $this->_rebuild_indexfile;
}

1;
