package Kumera::Print;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Print.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

use IO::File;
use File::Basename;

# these are the sections to look for in the html file for printing
my @NODES = qw/title body/;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    $class->_debug('type', $type);
    Kumera::_debug('class', $class);
    Kumera::_debug('INC', join ', ', @INC);

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ READ_ONLY => 1, %{ $args }});

    # overload the template filename from SUPER
    $this->{ TEMPLATE_FILENAME } = $this->_build_template_filename;
    $this->_debug('this->{ TEMPLATE_FILENAME }', $this->{ TEMPLATE_FILENAME });

    return bless $this, $type;
}

sub _build_template_filename {

    my $this = shift;
    my $args = shift;

    my $user_id = $args->{ USER_ID } || $this->{ USER_ID };
    my $path_info = $args->{ PATH_INFO } || $this->{ PATH_INFO };

    # need to loop through the directory structure
    # looking for the template that we want to use
    my $template_path = $path_info;

    while ($template_path ne '/' && $template_path ne '.') {

        $template_path = dirname($template_path);

        $template_filename = $this->getPrintFilename('root').($template_path ne '/' ? $template_path : '').$this->getPrintFilename('template');

        $this->_debug('template_filename', $template_filename);

        last if -r $template_filename;
    }

    $this->{ TEMPLATE_PATH_INFO } = $template_path.$this->getPrintFilename('template');


    return $template_filename;
}

sub getTemplatePathInfo {

    my $this = shift;

    return $this->{ TEMPLATE_PATH_INFO };
}


sub getPrintFile {

    my $this = shift;

    # read in the template file
    my $input = new IO::File($this->{ TEMPLATE_FILENAME });
    $input->untaint;
    my $template = join '', $input->getlines;
    $input->close;

    # create the hash that template expects
    my %text = ();
    my $print = '';

    # read in the browse file, may not exist?
    if (-r $this->{ BROWSE_FILENAME }) {
        $input->open($this->{ BROWSE_FILENAME });
        $input->untaint;
        $print = join '', $input->getlines;
        $input->close;
    }

    foreach $element (@NODES) {

        $text{$element} = $print =~ /<!-- start $element -->(.*?)<!-- end $element -->/s ? $1 : '';

        $this->_debug('text{'.$element.'}', $text{$element});
    }


    # now search for the [[ ]] bits to parse and execute
    $template =~ s/\[\[(.*?)\]\]/$1/eemg;

    return $template;
}

1;


