package Kumera::Upload;

####
##
## This code is distributed under the Open Source Artistic Licence
## http://www.opensource.org/licenses/artistic-license.html
##
## Kumera copyright (c) 2002, CyberCyberCyberCyber Pty Ltd.
## http://www.cyber4.org/kumera/index.html
##
## $Log: Upload.pm,v $
## Revision 1.1.1.1  2002/10/13 17:11:23  grumpy
## Added copyright.
##
##
####

require Exporter;
require Kumera::Base;
@ISA = qw/Exporter Kumera::Base/;
@EXPORT = qw//;
@EXPORT_OK = qw//;

use Net::FTP;
use File::Find;
use File::Basename;

sub new {

    # build object references
    my $type = shift;
    my $class = ref($type) || $type;

    # organise arguments
    my $args = shift;

    # run super's constructor, but with this type and args
    my $this = $class->SUPER::new({ %{ $args }});

    return bless $this, $type;
}

sub getUploadList {

    my $this = shift;

    # for output
    my @upload_list;
    my $path_info;

    # cache here
    my $root_filename = $this->getBrowseFilename('root');

    # open ftp connection to host site
    my $ftp = new Net::FTP($this->getParameter('host'));

    # log in
    $ftp->login($this->getParameter('username'), $this->getParameter('password'));

    # cache remote_root here
    my $remote_root = $this->getParameter('dir');

    $this->_debug('pwd', $ftp->pwd());

    find(
            {
                wanted => sub {

                    $this->_debug('found', $File::Find::name);

                    # skip unless it's an HTML file
                    return unless /\.html$/;

                    # strip the leading directory information from the front
                    $File::Find::name =~ m/^$root_filename(.*)\.html$/;
                    $path_info = "$1.html";

                    $this->_debug('File::Find::name', $File::Find::name);
                    $this->_debug('path_info', $path_info);

                    # do they have upload permission for this section
                    if ($this->hasPermission('upload', $path_info)) {

                        # need to check if this file is newer then what's on the
                        # server, or it may not exist on the server
                        push @upload_list, $path_info
                            if ((!defined $ftp->mdtm($remote_root.$path_info)) || (-M $root_filename.$path_info <= (time() - $ftp->mdtm($remote_root.$path_info)) / 86400));


                        $this->_debug('remote_file', $remote_root.$path_info);
                        $this->_debug('-M local file', -M $root_filename.$path_info);
                        $this->_debug('-M remote file', (time() - $ftp->mdtm($remote_root.$path_info)) / 86400) if defined $ftp->mdtm($remote_root.$path_info);
                        $this->_debug('action', (-M $root_filename.$path_info <= (utime() - $ftp->mdtm($remote_root.$path_info)) / 86400) ? "upload" : "ignore") if defined $ftp->mdtm($remote_root.$path_info);

                    }

                },
                untaint => 1,
                bydepth => 1
            },
            $root_filename
        );

    return sort @upload_list;
}

sub processUploadList {

    my $this = shift;
    my @upload_list = @_;

    # cache here
    my $browse_root = $this->getBrowseFilename('root');

    # the complete list
    my @complete_list;

    # open ftp connection to host site
    my $ftp = new Net::FTP($this->getParameter('host'));
    $ftp->login($this->getParameter('username'), $this->getParameter('password'));
    $ftp->ascii();

    # start with the root remote directory
    my $root_path = $this->getParameter('dir');
    my $remote_path = '';

    # loop through the files for upload
    foreach my $path_info (@upload_list) {

        $this->_debug('local_file', $browse_root.$path_info);
        $this->_debug('remote_file', $root_path.$path_info);

        # upload the file
        push @complete_list, $path_info if $ftp->put($browse_root.$path_info, $root_path.$path_info);

        # and change it's permission to 644 (may not work for some servers)
        $ftp->site("chmod 644 ".$root_path.$path_info);
    }

    return @complete_list;
}

1;
